@@ -22,10 +22,12 @@ from mongomock import helpers
 from mongomock import OperationFailure
 
 try:
+    from bson.errors import InvalidDocument
     from bson import Regex, decimal128
     decimal_support = True
     _RE_TYPES = (helpers.RE_TYPE, Regex)
 except ImportError:
+    InvalidDocument = OperationFailure
     decimal_support = False
     _RE_TYPES = (helpers.RE_TYPE)
 
@@ -180,9 +182,18 @@ def _sum_operation(values):
     return decimal128.Decimal128(sum_value) if isinstance(sum_value, decimal.Decimal) else sum_value
 
 
+def _merge_objects_operation(values):
+    merged_doc = dict()
+    for v in values:
+        if isinstance(v, dict):
+            merged_doc.update(v)
+    return merged_doc
+
+
 _GROUPING_OPERATOR_MAP = {
     '$sum': _sum_operation,
     '$avg': _avg_operation,
+    '$mergeObjects': _merge_objects_operation,
     '$min': lambda values: _group_operation(values, min),
     '$max': lambda values: _group_operation(values, max),
 }
@@ -267,6 +278,12 @@ class _Parser(object):
         except KeyError:
             return False
 
+    def _parse_or_nothing(self, expression):
+        try:
+            return self.parse(expression)
+        except KeyError:
+            return NOTHING
+
     def _parse_basic_expression(self, expression):
         if isinstance(expression, six.string_types) and expression.startswith('$'):
             if expression.startswith('$$'):
@@ -344,10 +361,12 @@ class _Parser(object):
 
     def _handle_project_operator(self, operator, values):
         if operator in _GROUPING_OPERATOR_MAP:
-            return _GROUPING_OPERATOR_MAP[operator](self.parse_many(values))
+            values = self.parse(values) if isinstance(values, str) else self.parse_many(values)
+            return _GROUPING_OPERATOR_MAP[operator](values)
         if operator == '$arrayElemAt':
             key, index = values
             array = self._parse_basic_expression(key)
+            index = self.parse(index)
             return array[index]
         raise NotImplementedError("Although '%s' is a valid project operator for the "
                                   'aggregation pipeline, it is currently not implemented '
@@ -356,6 +375,23 @@ class _Parser(object):
     def _handle_projection_operator(self, operator, value):
         if operator == '$literal':
             return value
+        if operator == '$let':
+            if not isinstance(value, dict):
+                raise InvalidDocument('$let only supports an object as its argument')
+            for field in ('vars', 'in'):
+                if field not in value:
+                    raise OperationFailure("Missing '{}' parameter to $let".format(field))
+            if not isinstance(value['vars'], dict):
+                raise OperationFailure('invalid parameter: expected an object (vars)')
+            user_vars = {
+                var_key: self.parse(var_value)
+                for var_key, var_value in six.iteritems(value['vars'])
+            }
+            return _Parser(
+                self._doc_dict,
+                dict(self._user_vars, **user_vars),
+                ignore_missing_keys=self._ignore_missing_keys,
+            ).parse(value['in'])
         raise NotImplementedError("Although '%s' is a valid project operator for the "
                                   'aggregation pipeline, it is currently not implemented '
                                   'in Mongomock.' % operator)
@@ -566,10 +602,11 @@ class _Parser(object):
                     raise OperationFailure('Expression $size takes exactly 1 arguments. '
                                            '%d were passed in.' % len(value))
                 value = value[0]
-            array_value = self.parse(value)
-            if not isinstance(array_value, list):
-                raise OperationFailure('The argument to $size must be an array, '
-                                       'but was of type: %s' % type(array_value))
+            array_value = self._parse_or_nothing(value)
+            if not isinstance(array_value, (list, tuple)):
+                raise OperationFailure(
+                    'The argument to $size must be an array, but was of type: %s' %
+                    ('missing' if array_value is NOTHING else type(array_value)))
             return len(array_value)
 
         if operator == '$filter':
@@ -656,6 +693,19 @@ class _Parser(object):
                 'You need to import the pymongo library to support decimal128 type.'
             )
 
+        if operator == '$toLong':
+            try:
+                parsed = self.parse(values)
+            except KeyError:
+                return None
+            if decimal_support:
+                if isinstance(parsed, decimal128.Decimal128):
+                    return helpers.to_long(parsed.to_decimal())
+                return helpers.to_long(parsed)
+            raise NotImplementedError(
+                'You need to import the pymongo library to support decimal128 type.'
+            )
+
         # Document: https://docs.mongodb.com/manual/reference/operator/aggregation/toDecimal/
         if operator == '$toDecimal':
             if not decimal_support:
@@ -835,6 +885,9 @@ def _parse_expression(expression, doc_dict, ignore_missing_keys=False):
     return _Parser(doc_dict, ignore_missing_keys=ignore_missing_keys).parse(expression)
 
 
+filtering.register_parse_expression(_parse_expression)
+
+
 def _accumulate_group(output_fields, group_list):
     doc_dict = {}
     for field, value in six.iteritems(output_fields):
@@ -1298,7 +1351,7 @@ def _handle_project_stage(in_collection, unused_database, options):
             except KeyError:
                 # Ignore missing key.
                 pass
-    if (method == 'include') == (include_id is not False and include_id is not 0):
+    if (method == 'include') == (include_id is not False and include_id != 0):
         filter_list.append('_id')
 
     if not filter_list:
