@@ -56,7 +56,6 @@ from mongomock import InvalidOperation
 from mongomock.not_implemented import raise_for_feature as raise_not_implemented
 from mongomock import ObjectId
 from mongomock import OperationFailure
-from mongomock.read_concern import ReadConcern
 from mongomock.results import BulkWriteResult
 from mongomock.results import DeleteResult
 from mongomock.results import InsertManyResult
@@ -65,6 +64,11 @@ from mongomock.results import UpdateResult
 from mongomock.write_concern import WriteConcern
 from mongomock import WriteError
 
+try:
+    from pymongo.read_concern import ReadConcern
+except ImportError:
+    from mongomock.read_concern import ReadConcern
+
 if hasattr(time, 'perf_counter'):
     _get_perf_counter = time.perf_counter
 else:
@@ -79,9 +83,6 @@ _WITH_OPTIONS_KWARGS = {
     'write_concern': _KwargOption(
         'pymongo.write_concern.WriteConcern', WriteConcern(),
         ('acknowledged', 'document')),
-    'read_concern': _KwargOption(
-        'pymongo.read_concern.ReadConcern', ReadConcern(),
-        ('document', 'level', 'ok_for_legacy'))
 }
 
 
@@ -380,6 +381,8 @@ class Collection(object):
         self._name = name
         self._db_store = _db_store
         self._write_concern = write_concern or WriteConcern()
+        if read_concern and not isinstance(read_concern, ReadConcern):
+            raise TypeError('read_concern must be an instance of pymongo.read_concern.ReadConcern')
         self._read_concern = read_concern or ReadConcern()
         self._read_preference = read_preference or _READ_PREFERENCE_PRIMARY
         self._codec_options = codec_options or mongomock_codec_options.CodecOptions()
@@ -1206,8 +1209,7 @@ class Collection(object):
                         current_doc = current_doc[part]
 
                     subdocument = current_doc
-                    if (field_name_parts[-1] == '$' and
-                            isinstance(subdocument, list)):
+                    if field_name_parts[-1] == '$' and isinstance(subdocument, list):
                         for i, doc in enumerate(subdocument):
                             if filter_applies(subspec, doc):
                                 subdocument[i] = v
